#!/usr/bin/env python
"""
theme.py, module definition of Theme class.
This defines the theme of the presentation.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# MaTiSSe.py modules
from .box import Box
from .canvas import Canvas
from .figure import Figure
from .heading import Heading
from .note import Note
from .selector import Selector
from .slide.slide import Slide
from .table import Table
from .toc import TOC
from ..utils.source_editor import __source_editor__
# global variables
# default theme settings
__p_css__ = {'p': '\n  padding: 1% 0;'}
__a_css__ = {'a': '\n  color: inherit;\n  text-decoration: none;\n  transition: 0.5s;',
             'a:hover, a:focus': '\n  background: rgba(255,255,255,1);\n  text-shadow: -1px -1px 2px rgba(100,100,100,0.5);'}
__ul_css__ = {'ul': '\n  padding-left: 2em;'}
__ol_css__ = {'ol': '\n  padding-left: 2em;'}
__pre_css__ = {'pre': '\n  display:flex;\n  padding: 1% 0;'}
__columns_css__ = {'.columns:after': '\n  visibility: hidden;\n  content:" ";\n  clear: both;\n display: block;'}
__input_btn_css__ = {'input[type=button]': '\n  padding:0;\n  margin:0;\n  border:0 none;\n  background: rgba(0,0,0,0.01);'}
__countDown_css__ = {'.countDown': '\n  display: block;\n  padding: 0;\n  margin: 0;'}
__timer_css__ = {'.timer': '\n  display:table-row;',
                 '.timercontrols': '\n  display:table-row;'}
__impress_css__ = {'.step': '\n  position: relative;',
                   '.impress-enabled .step': '\n  margin: 0; \n  opacity: 0.3; \n  -webkit-transition: opacity 1s; \n  -moz-transition: opacity 1s; \n  -ms-transition: opacity 1s; \n  -o-transition: opacity 1s; \n  transition: opacity 1s;',
                   '.impress-enabled .step.active': '\n  opacity: 1;'}
__table_css__ = {'table tr': 'border-top: 1px solid #cccccc; background-color: white; margin: 0; padding: 0;',
                 'table tr:nth-child(2n)': 'background-color: #f8f8f8;',
                 'table tr th': 'font-weight: bold; border: 1px solid #cccccc; text-align: left; margin: 0; padding: 6px 13px;',
                 'table tr td': ' border: 1px solid #cccccc; text-align: left; margin: 0; padding: 6px 13px;',
                 'table tr th :first-child, table tr td :first-child': 'margin-top: 0;',
                 'table tr th :last-child, table tr td :last-child': 'margin-bottom: 0;'}
__default_css__ = ['*, *:after, *:before {\n  -webkit-box-sizing: border-box;\n  -moz-box-sizing: border-box;\n  box-sizing: border-box;\n  margin: 0;\n  padding: 0;\n}']
__default_css__.append(''.join(['\n' + sel + ' {' + __p_css__[sel] + '\n}' for sel in __p_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __a_css__[sel] + '\n}' for sel in __a_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __ul_css__[sel] + '\n}' for sel in __ul_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __ol_css__[sel] + '\n}' for sel in __ol_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __pre_css__[sel] + '\n}' for sel in __pre_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __columns_css__[sel] + '\n}' for sel in __columns_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __input_btn_css__[sel] + '\n}' for sel in __input_btn_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __countDown_css__[sel] + '\n}' for sel in __countDown_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __timer_css__[sel] + '\n}' for sel in __timer_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __impress_css__[sel] + '\n}' for sel in __impress_css__]))
__default_css__.append(''.join(['\n' + sel + ' {' + __table_css__[sel] + '\n}' for sel in __table_css__]))
__default_css__ = ''.join(__default_css__)


# class definition
class Theme(object):
  """Theme is an object that handles the presentation theme, its attributes and methods."""
  def __init__(self, source=None, defaults=False, set_all_custom=False, reset=False):
    """
    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    defaults : bool, optional
      flag for activatin the creation of a presentation istance
      having one of each element available with the default
      settings
    set_all_custom : bool, optional
      set all option as customized from users (useful for plain slides theme)
    reset : bool, optional
      reset theme components

    Attributes
    ----------
    canvas : Canvas object
      convas over which all elements are rendered
    headings : list
      list of Heading objects for handling style of html headings (h1,h2,...h6)
    titlepage: none
    slide : Slide object
      base theme of slides
    selectors : list
      list of Selector objects for handling user defined custom selectors (e.g. <code> tags)
    toc : TOC object
      Table of Contents theme

    >>> source =  '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> source += '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> theme = Theme(source=source)
    >>> theme.slide.headers['slide-header_1'].data.data['height'][0]
    '10%'
    >>> theme = Theme(defaults=True)
    >>> theme.slide.has_header()
    (True, 1)
    >>> theme = Theme(defaults=True,set_all_custom=True)
    >>> theme.slide.headers['slide-header_1'].data.data['height'][1]
    True
    """
    if reset:
      self.__reset()
    self.canvas = Canvas()
    self.headings = []
    for hds in range(6):
      self.headings.append(Heading(number=hds + 1))
    self.slide = Slide(defaults=defaults)
    self.selectors = []
    self.toc = TOC()
    if source:
      self.get(source=__source_editor__.purge_overtheme(source))
    elif defaults:
      self.selectors.append(Selector(name='def-sel'))
    if set_all_custom:
      self.canvas.set_all_custom()
      for hds in self.headings:
        hds.set_all_custom()
      self.slide.set_all_custom()
      for sel in self.selectors:
        sel.set_all_custom()
    return

  def __str__(self):
    string = 'Theme settings\n'
    string += '  Canvas\n'
    string += str(self.canvas)
    for hds, heading in enumerate(self.headings):
      string += '\n  h' + str(hds + 1) + '\n'
      string += str(heading)
    string += str(self.slide)
    for selector in self.selectors:
      string += '\n  selector ' + selector.name + '\n'
      string += str(selector)
    string += '\n  TOC\n'
    string += str(self.toc)
    return string

  def __get_selectors(self, source):
    """Method for getting the definition of user-defined custom selectors.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    number = Selector.count(source)
    if number > 0:
      sel_regex = re.compile(r'[-]{3}theme_selector_(?P<name>[a-zA-Z][a-zA-Z0-9_\-]*)')
      names = []
      for match in re.finditer(sel_regex, source):
        names.append(match.group('name'))
      for name in names:
        self.selectors.append(Selector(name=name, source=source))
    return

  @staticmethod
  def __reset():
    """Method resetting theme to initial values."""
    Box.reset()
    Figure.reset()
    Table.reset()
    Note.reset()
    return

  def check_specials(self):
    """Method for checking specials data entries.
    The main theme has not special entries, not being a subclass of ThemeElement.
    However the check_specials method of other contained theme elements is called.
    """
    self.canvas.check_specials()
    for hds in self.headings:
      hds.check_specials()
    self.slide.check_specials()
    if len(self.selectors) > 0:
      for selector in self.selectors:
        selector.check_specials()
    return

  def get(self, source):
    """Method for getting data values from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    self.canvas.get(source=source)
    for hds in self.headings:
      hds.get(source=source)
    self.slide.get(source=source)
    self.__get_selectors(source=source)
    self.toc.get(source=source)
    self.check_specials()
    Figure.get_theme(source=source)
    Note.get_theme(source=source)
    Table.get_theme(source=source)
    return

  def set_from(self, other):
    """Method for setting theme using data of other theme.

    Parameters
    ----------
    other : Theme object

    >>> source =  '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> source += '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_sidebar_1 width = 10% ---endtheme_slide_sidebar_1'
    >>> thm1 = Theme(source=source)
    >>> thm1.slide.headers['slide-header_1'].data.data['background'] = ['white',False]
    >>> source =  '---theme_slide_header_1 background = red ---endtheme_slide_header_1'
    >>> source += '---theme_slide_header_2 background = red ---endtheme_slide_header_2'
    >>> source += '---theme_slide_footer_1 background = green ---endtheme_slide_footer_1'
    >>> source += '---theme_slide_footer_2 background = green ---endtheme_slide_footer_2'
    >>> source += '---theme_slide_sidebar_1 background = blue ---endtheme_slide_sidebar_1'
    >>> source += '---theme_slide_sidebar_2 background = blue ---endtheme_slide_sidebar_2'
    >>> thm2 = Theme(source=source)
    >>> thm1.set_from(thm2)
    >>> thm1.slide.headers['slide-header_1'].data.data['background'][0]
    'red'
    """
    self.canvas.set_from(other=other.canvas)
    for hds, head in enumerate(self.headings):
      head.set_from(other=other.headings[hds])
    self.slide.set_from(other=other.slide)
    self.toc.set_from(other=other.toc)
    self.check_specials()
    return

  # def update(self,source):
  #  """Method for updating data from source without creating new data.

  #  Parameters
  #  ----------
  #  source : str
  #    string (as single stream) containing the source
  #  """
  #  self.canvas.get(source=source)
  #  for hds in self.headings:
  #    hds.get(source=source)
  #  self.slide.update(source=source)
  #  if len(self.selectors)>0:
  #    for slr in self.selectors:
  #      slr.get(source=source)
  #  self.toc.get(source=source)
  #  self.check_specials()
  #  return

  def get_custom(self, chk_specials=False):
    """Method returning only the data that have been set by users (customized) overriding default values.

    Parameters
    ----------
    chk_specials : bool, optional
      if activated handle special entries differently from standard ones

    Returns
    -------
    dict
      dictionary of customized data

    >>> thm1 = Theme(defaults=True)
    >>> thm1.slide.headers['slide-header_1'].data.data['background'] = ['white',True]
    >>> thm1.get_custom()['slide-header_1'].data['background'][0]
    'white'
    """
    custom = self.slide.get_custom(chk_specials=chk_specials)
    custom['canvas'] = self.canvas.data.get_custom(chk_specials=chk_specials)
    for hds in self.headings:
      custom['h' + str(hds.number)] = hds.data.get_custom(chk_specials=chk_specials)
    if len(self.selectors) > 0:
      for selector in self.selectors:
        custom['slide-selector_' + selector.name] = selector.data.get_custom(chk_specials=chk_specials)
    custom['toc'] = self.toc.data.get_custom(chk_specials=chk_specials)
    return custom

  def get_options(self):
    """Method for getting the available data options.

    Returns
    -------
    str
      string with option_names = values pairs (without True/False custom tag)

    >>> thm1 = Theme(defaults=True)
    >>> thm1.get_options().split('\\n')[4]
    'active = True'
    """
    string = []
    string.append(self.canvas.get_options())
    for hds in self.headings:
      string.append(hds.get_options())
    string.append(self.slide.get_options())
    for sls in self.selectors:
      string.append(sls.get_options())
    string.append(self.toc.get_options())
    return ''.join(string)

  def get_css(self, only_custom=False, as_list=False):
    """Method for creating the css theme. The returned string contains the css theme.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the theme if as_list = False
    list
      a list of one string containing the css code of the theme if as_list = True

    >>> thm1 = Theme(defaults=True)
    >>> thm1.get_css(as_list=True)[0].split('\\n')[4]
    '  margin: 0;'
    """
    css = [__default_css__]
    css.append(self.canvas.get_css(only_custom=only_custom))
    for hds in self.headings:
      css.append(hds.get_css(only_custom=only_custom))
    css.append(self.slide.get_css(only_custom=only_custom))
    for sls in self.selectors:
      css.append(sls.get_css(only_custom=only_custom))
    css.append(self.toc.get_css(only_custom=only_custom))
    if as_list:
      return css
    else:
      return ''.join(css)

  def strip(self, source):
    """Method for striping theme raw data from source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source

    Returns
    -------
    str
      source without the theme data
    """
    strip_source = self.canvas.strip(source)
    if len(self.headings) > 0:
      for hds in self.headings:
        strip_source = hds.strip(strip_source)
    strip_source = self.slide.strip(strip_source)
    if len(self.selectors) > 0:
      for sls in self.selectors:
        strip_source = sls.strip(strip_source)
    strip_source = self.toc.strip(strip_source)
    # strip_source = box_strip_themes(strip_source)
    strip_source = Figure.strip_theme(strip_source)
    strip_source = Note.strip_theme(strip_source)
    strip_source = Table.strip_theme(strip_source)
    return strip_source.strip()



"""md_mathjax is a simple extension of the Python implementation of Markdown

author: Man YUAN
homepage: https://github.com/epsilony/md_mathjax
"""
from markdown.util import AtomicString
from markdown.util import etree
from markdown.inlinepatterns import Pattern
from markdown import Extension

class MathJaxPattern(Pattern):
  groups = 2, 3, 4
  start_end = None

  def __init__ (self, start_end=None, groups=None):
    if start_end is not None:
      self.start_end = start_end
    if groups is not None:
      self.groups = groups
    pattern = r'(?<!\\)(%s)(.+?)(?<!\\)(%s)' % (self.start_end)
    Pattern.__init__(self, pattern)

  def handleMatch(self, m):
    node = etree.Element(None)
    text = ''
    for group in self.groups:
      text += m.group(group)
    node.text = AtomicString(text)
    return node

class MathJaxInlinePattern(MathJaxPattern):
  start_end = r'\\\(', r'\\\)'

class BraketPattern(MathJaxPattern):
  start_end = r'\\\[', r'\\\]'

class DoubleDollarPattern(MathJaxPattern):
  start_end = r'\$\$', r'\$\$'

class BeginEndPattern(MathJaxPattern):
  start_end = r'\\begin\{(.+?)\}', r'\\end\{\3\}'
  groups = 2, 4, 5

class MathJaxExtension(Extension):
  def extendMarkdown(self, md, md_globals):
    md.inlinePatterns.add('mathjax_invironment', BeginEndPattern(), '<escape')
    md.inlinePatterns.add('mathjax_bracket', BraketPattern(), '<escape')
    md.inlinePatterns.add('mathjax_double_dollar', DoubleDollarPattern(), '<escape')
    md.inlinePatterns.add('mathjax_inline', MathJaxInlinePattern(), '<escape')

def makeExtension(configs=None):
  return MathJaxExtension(configs)



#!/usr/bin/env python
"""
columns.py, module definition of Columns class.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import re
# modules not in the standard library
from yattag import Doc
# MaTiSSe.py modules
from .box    import parse as box_parse
from .figure import parse as figure_parse
from .note   import parse as note_parse
from .table  import parse as table_parse
from ..utils.source_editor import __source_editor__ as seditor
from ..utils.source_editor import obfuscate_codeblocks as obfuscate
from ..utils.source_editor import illuminate_protected as illuminate
from ..utils.source_editor import tokenize as generic_tokenize
# global variables
# regular expressions
__recolumns__ = re.compile(r"(?P<columns>\$columns(?P<env>.*?)\$endcolumns)",re.DOTALL)
__recol__ = re.compile(r"(?P<column>\$column(\[(?P<options>.*?)\])*)",re.DOTALL)
# classes definition
class Columns(object):
  """
  Object for handling columns environment.

  The syntax is:

  $columns
  $column[width:30%;background:green;]
  the first column contents
  $column[width:60%;background:white;]
  the second column contents
  $column[width:10%;background:red;]
  the third column contents
  $endcolumns
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      columns number
    columns : list
      list of list containing contents and options [cont,opts] of columns
    """
    self.number   = 0
    self.columns  = []
    if source:
      self.get(source=source)
    return

  def __str__(self):
    string = []
    if self.number > 0:
      string.append('\nColumns number: '+str(self.number))
      for col,column in enumerate(self.columns):
        string.append('\nColumn('+str(col)+')[options='+str(column[1])+']')
    return ''.join(string)

  def get(self,source=''):
    """Method for getting box style data.

    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source
    """
    protected, obfuscate_source = obfuscate(source = source)
    columns = []
    for match in re.finditer(__recol__,obfuscate_source):
      columns.append([match.group('options'),match.start(),match.end()])
    if len(columns)>0:
      for col,column in enumerate(columns):
        if col < len(columns)-1:
          contents = obfuscate_source[column[2]+1:columns[col+1][1]]
        else:
          contents = obfuscate_source[column[2]+1:]
        self.columns.append([illuminate(source=contents,protected_contents=protected),column[0]])
    self.number = len(self.columns)
    return

  def to_html(self):
    """Method for inserting columns to the html doc."""
    if self.number > 0:
      doc = Doc()
      with doc.tag('div',klass='columns'):
        for col,column in enumerate(self.columns):
          with doc.tag('div',klass='column'):
            doc.attr(('column-number',str(col+1)))
            style = 'display:block;float:left;'
            if column[1]:
              style += column[1]
            else:
              style += 'width:'+str(int(100.0/self.number))+'%;'
            doc.attr(style=style)
            content = box_parse(column[0])
            content = figure_parse(content)
            content = table_parse(content)
            content = note_parse(content)
            doc.asis(seditor.md_convert(content))
      return doc.getvalue()
    return ''

def tokenize(source):
  """Method for tokenizing source tagging columns environments.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  list
    list of tokens whose elements are ['type',source_part]; type is 'columns' for columns environments and
    'unknown' for anything else
  """
  return generic_tokenize(source=source,re_part=__recolumns__,name_part='columns')

def parse(source):
  """Method for parsing source substituting boxes with their own html equivalent.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source

  Returns
  -------
  str
    source string parsed
  """
  protected, obfuscate_source = obfuscate(source = source)
  for match in re.finditer(__recolumns__,obfuscate_source):
    columns = Columns(source=illuminate(source=match.group('env'),protected_contents=protected))
    obfuscate_source = re.sub(__recolumns__,columns.to_html(),obfuscate_source,1)
  return illuminate(source=obfuscate_source,protected_contents=protected)



#!/usr/bin/env python
"""Setup script for install MaTiSSe.py"""
import os
import re
from setuptools import setup
__data_files_builtin_themes__ = [os.path.join(dp, f) for dp, dn, filenames in os.walk('matisse/utils/builtin_themes/') for f in filenames]
__data_files_css__ = [os.path.join(dp, f) for dp, dn, filenames in os.walk('matisse/utils/css/') for f in filenames]
__data_files_js__ = [os.path.join(dp, f) for dp, dn, filenames in os.walk('matisse/utils/js/') for f in filenames]
__data_files_pairs__ = []
for data in __data_files_builtin_themes__:
  __data_files_pairs__.append((os.path.dirname(data), [data]))
for data in __data_files_css__:
  __data_files_pairs__.append((os.path.dirname(data), [data]))
for data in __data_files_js__:
  __data_files_pairs__.append((os.path.dirname(data), [data]))

__source__ = open('matisse/matisse.py').read()
__license__ = re.search(r'^__license__\s*=\s*"(.*)"', __source__, re.M).group(1)

if __name__ == '__main__':
  setup(name=re.search(r'^__appname__\s*=\s*"(.*)"', __source__, re.M).group(1),
        version=re.search(r'^__version__\s*=\s*"(.*)"', __source__, re.M).group(1),
        description=re.search(r'^__description__\s*=\s*"(.*)"', __source__, re.M).group(1),
        long_description=re.search(r'^__long_description__\s*=\s*"(.*)"', __source__, re.M).group(1),
        author=re.search(r'^__author__\s*=\s*"(.*)"', __source__, re.M).group(1),
        author_email=re.search(r'^__author_email__\s*=\s*"(.*)"', __source__, re.M).group(1),
        url=re.search(r'^__url__\s*=\s*"(.*)"', __source__, re.M).group(1),
        scripts=['MaTiSSe.py'],
        packages=['matisse', 'matisse.data', 'matisse.presentation', 'matisse.utils', 'matisse.theme', 'matisse.theme.slide'],
        py_modules=[],
        classifiers=['Development Status :: 5 - Production/Stable',
                     'License :: OSI Approved :: ' + __license__,
                     'Environment :: Console',
                     'Intended Audience :: End Users/Desktop',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 2',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3',
                     'Programming Language :: Python :: 3.4',
                     'Topic :: Text Processing'],
        entry_points={'console_scripts': []},
        package_data={'': ['*.md', '*.css', '*.js']},
        data_files=__data_files_pairs__,
        include_package_data=True,
        install_requires=["markdown", "yattag"])



#!/usr/bin/env python
"""
selector.py, module definition of Selector class.
This defines the theme of a generic "entity" selector that can be
customized by the user.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from .theme_element import ThemeElement
# class definition
class Selector(ThemeElement):
  """
  Object for handling a generic entity selector, its attributes and methods.
  This can be useful for user customized entities.

  Note that for supporting nested css selectors the selector name can use
  the "-" symbol to compose nested selectors, e.g.:
  name = 'pre-span' defines a nested selector for the <pre><span>...</span></pre>
  selectors resulting in a css like "pre span {...}".
  """
  data_tag = r'theme_selector_'

  @classmethod
  def count(cls,source):
    """Method for computing the number of custom selectors defined into the source.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    sel = ThemeElement(data_tag=Selector.data_tag)
    return sel.data.count(source)

  def __init__(self,name,source=None):
    self.name = name
    super(Selector,self).__init__(data_tag=Selector.data_tag+self.name)
    self.data.data['display'        ] = ['inherit',False]
    self.data.data['margin'         ] = ['inherit',False]
    self.data.data['padding'        ] = ['inherit',False]
    self.data.data['background'     ] = ['inherit',False]
    self.data.data['color'          ] = ['inherit',False]
    self.data.data['font'           ] = ['inherit',False]
    self.data.data['font-weight'    ] = ['inherit',False]
    self.data.data['font-size'      ] = ['inherit',False]
    self.data.data['font-family'    ] = ['inherit',False]
    self.data.data['text-decoration'] = ['inherit',False]
    self.data.data['text-transform' ] = ['inherit',False]
    self.data.data['text-shadow'    ] = ['inherit',False]
    self.data.data['letter-spacing' ] = ['inherit',False]
    self.data.data['line-height'    ] = ['inherit',False]
    self.data.data['border'         ] = ['inherit',False]
    self.data.data['border-radius'  ] = ['inherit',False]
    self.data.data['box-shadow'     ] = ['inherit',False]
    self.data.data['white-space'    ] = ['inherit',False]
    self.data.data['overflow-x'     ] = ['inherit',False]
    self._name = self.name.replace('-',' ')
    if source:
      self.get(source)
    return

  def get_options(self):
    """Method for getting the available data options."""
    string = ['\n\nSelector '+self._name]
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.
    """
    css = "\n"+self._name+" {"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css



#!/usr/bin/env python
"""
config.py, module definition of MaTiSSe.py configuration.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
import os
# import re
from shutil import copytree, rmtree
import sys
# global variables
# __regex_builtin_themes__ = re.compile(r"\$builtin_themes" + os.sep)


# class definition
class MaTiSSeConfig(object):
  """
  Object handling MaTiSSe.py configuration

  Attributes
  ----------
  __highlight_styles : list
    list of available highlight.js styles
  __themes : list
    list of builtin themes
  """
  __highlight_styles = []
  __themes = []

  def __init__(self):
    """
    Attributes
    ----------
    verbose : bool
      more verbose printing messages (default no)
    indented : bool
      indent html output file (default no, may corrupt TOC rendering)
    online_mathjax : bool
      use online rendering of LaTeX equations by means of online MathJax service;
      default no, use offline local copy of MathJax engine
    highlight : bool
      use highlight.js for syntax highlithing of code blocks
    highlight_style : str
      css style file for highlight.js; the list of available styles can be print by
      'str_highlight_styles' method
    theme : str
      builtin theme chosen
    toc_at_sec_beginning : bool
      insert a slide with TOC at the beginning of each section (default false)
    toc_at_subsec_beginning : bool
      insert a slide with TOC at the beginning of each subsection (default false)
    """
    self.verbose = False
    self.indented = False
    self.online_mathjax = False
    self.highlight = True
    self.highlight_style = 'github.css'
    self.theme = None
    self.toc_at_sec_beginning = None
    self.toc_at_subsec_beginning = None
    self.__get_highlight_styles()
    self.__check_highlight_style()
    self.__get_themes()
    self.__check_theme()
    if self.verbose:
      print(self)
    return

  def __str__(self):
    string = ['MaTiSSe.py configuration']
    string.append('\n  Verbose mode: ' + str(self.verbose))
    string.append('\n  Indent html output: ' + str(self.indented))
    if self.online_mathjax:
      string.append('\n  LaTeX equations rendering by means of online MathJax service')
    else:
      string.append('\n  LaTeX equations rendering by means offline, local copy of MathJax')
    if self.highlight:
      string.append('\n  Highlight.js style: ' + self.highlight_style)
    string.append('\n  Insert TOC at sections beginning: ' + str(self.toc_at_sec_beginning))
    string.append('\n  Insert TOC at subsections beginning: ' + str(self.toc_at_subsec_beginning))
    return ''.join(string)

  @staticmethod
  def __get_highlight_styles():
    """Get the available highlight.js styles."""
    hilite_styles = os.path.join(os.path.dirname(__file__), 'utils/js/highlight/styles')
    for css in os.listdir(hilite_styles):
      if css.endswith(".css"):
        MaTiSSeConfig.__highlight_styles.append(css)
    return

  @staticmethod
  def __get_themes():
    """Get the builtin themes."""
    themes = os.path.join(os.path.dirname(__file__), 'utils/builtin_themes')
    for theme in os.listdir(themes):
      MaTiSSeConfig.__themes.append(theme)
    return

  def __check_highlight_style(self):
    """Check if the selected highlight.js style is available."""
    if self.highlight_style != 'disable':
      avail = (self.highlight_style in MaTiSSeConfig.__highlight_styles)
      if not avail:
        sys.stderr.write("Error: the selected highlight.js style '" + self.highlight_style + "' is not available")
        sys.stderr.write("\nRestore the default value 'github.css'\n")
        self.highlight_style = 'github.css'
        sys.stderr.write(self.str_highlight_styles())
    else:
      avail = False
      self.highlight = False
    return avail

  def __check_theme(self):
    """Check if the selected builtin theme is available."""
    avail = False
    if self.theme:
      avail = (self.theme in MaTiSSeConfig.__themes)
      if not avail:
        self.theme = None
        sys.stderr.write("Error: the selected builtin theme '" + self.theme + "' is not available")
        sys.stderr.write(self.str_themes())
    return avail

  def set_highlight_style(self, style):
    """Set highlight.js style performing availability check.

    Parameters
    ----------
    style : str
      style file name
    """
    self.highlight_style = style
    self.__check_highlight_style()
    return

  def set_theme(self, theme):
    """Set builtin theme performing availability check.

    Parameters
    ----------
    theme : str
      theme file name
    """
    self.theme = theme
    self.__check_theme()
    return

  def put_theme(self, source):
    """Put builtin theme into the source.

    Parameters
    ----------
    source : str
      source of presentation

    Returns
    -------
    str
      source of presentation with theme included
    """
    source_themed = source
    if self.theme:
      themes = os.path.join(os.path.dirname(__file__), 'utils/builtin_themes')
      for theme in os.listdir(themes):
        if theme == self.theme:
          if os.path.exists(theme):
            rmtree(theme)
          copytree(themes + os.sep + theme, theme)
          source_themed = '$include(' + theme + os.sep + 'theme.md)\n' + source

          # with open(themes + os.sep + theme + os.sep + 'theme.md', 'r') as themefile:
          #   theme_source = themefile.read()
          # source_themed = theme_source + '\n' + source
          # if __regex_builtin_themes__.search(source_themed):
          #   source_themed = __regex_builtin_themes__.sub('', source_themed)
    return source_themed

  def str_highlight_styles(self):
    """Method for stringify the available highlight.js styles.

    Returns
    -------
    str
      string containing the list of available styles
    """
    string = ['Available highlight.js styles']
    for style in self.__highlight_styles:
      string.append(style)
    return '\n  '.join(string) + '\n'

  def str_themes(self):
    """Stringify the builtin themes.

    Returns
    -------
    str
      string containing the list of builtin themes
    """
    string = ['Builtin themes']
    for theme in self.__themes:
      string.append(theme)
    return '\n  '.join(string) + '\n'

  def update(self, cliargs):
    """Method for updating config state from command line arguments.

    Parameters
    ----------
    cliargs : argparse parsed object
      command line arguments parsed
    """
    self.verbose = cliargs.verbose
    self.indented = cliargs.indented
    self.online_mathjax = cliargs.online_MathJax
    self.set_highlight_style(style=cliargs.highlight_style)
    self.set_theme(theme=cliargs.theme)
    self.toc_at_sec_beginning = cliargs.toc_at_sec_beginning
    self.toc_at_subsec_beginning = cliargs.toc_at_subsec_beginning
    return

  def printf(self):
    """Method for printing config data. It checks verbosity."""
    if self.verbose:
      print(self)
    return

# global variables
__initialized__ = False
if not __initialized__:
  __config__ = MaTiSSeConfig()
  __initialized__ = True



#!/usr/bin/env python
"""
content.py, module definition of Content class.
This defines the theme of the slide content element.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from ..theme_element import ThemeElement
# class definition
class Content(ThemeElement):
  """
  Object for handling the presentation theme slide content, its attributes and methods.
  """
  def __init__(self,source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source
    """
    super(Content,self).__init__(data_tag=r'theme_slide_content',class_name='slide-content')
    self.data.data['display'    ] = ['block',                       False]
    self.data.data['width'      ] = ['100%',                        False]
    self.data.data['height'     ] = ['100%',                        False]
    self.data.data['padding'    ] = ['0',                           False]
    self.data.data['font-size'  ] = ['100%',                        False]
    self.data.data['font-family'] = ['Open Sans, Arial, sans-serif',False]
    self.data.data['overflow'   ] = ['hidden',                      False]
    if source:
      self.get(source)
      self.check_specials()
    return

  def adjust_dims(self,headers,footers,sidebars):
    """Method for adjusting content dimensions accordingly to the settings of other elements of the slide theme.

    Parameters
    ----------
    headers : dict
      dictionary with values of Header objects, being the list of headers defined into the slide
    footers : dict
      dictionary with values of Footer objects, being the list of footers defined into the slide
    sidebars : dict
      dictionary with values of Sidebar objects, being the list of sidebars defined into the slide

    >>> from .header import Header
    >>> source = '---theme_slide_header_1 height = 10% ---endtheme_slide_header_1'
    >>> myheader = Header(number=1,source=source)
    >>> source = '---theme_slide_content background = red ---endtheme_slide_content'
    >>> mycontent = Content(source=source)
    >>> mycontent.adjust_dims(headers={'1':myheader},footers={},sidebars={})
    >>> mycontent.data.data['height'][0]
    '90%'
    """
    s_w = 100 # slide content width (in percent %); should be always 100% initially
    s_h = 100 # slide content height (in percent %); should be always 100% initially
    if len(headers)>0:
      for header in headers.values():
        if header.active:
          s_h -= int(header.data.data['height'][0].strip('%'))
    if len(footers)>0:
      for footer in footers.values():
        if footer.active:
          s_h -= int(footer.data.data['height'][0].strip('%'))
    if len(sidebars)>0:
      for sidebar in sidebars.values():
        if sidebar.active:
          s_w -= int(sidebar.data.data['width'][0].strip('%'))
    self.data.data['width']  = [str(s_w)+'%',True]
    self.data.data['height'] = [str(s_h)+'%',True]
    return

  def get_options(self):
    """Method for getting the available data options.

    >>> source = '---theme_slide_content background = red ---endtheme_slide_content'
    >>> mycontent = Content(source=source)
    >>> mycontent.get_options()
    '\\n\\nSlide Content\\nmetadata = []\\nactive = True\\ndisplay = block\\nwidth = 100%\\nheight = 100%\\npadding = 0\\nfont-size = 100%\\nfont-family = Open Sans, Arial, sans-serif\\noverflow = hidden\\nbackground = red'
    """
    string = ['\n\nSlide Content']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the theme if as_list = False
    list
      a list of one string containing the css code of the theme if as_list = True

    >>> source = '---theme_slide_content background = red ---endtheme_slide_content'
    >>> mycontent = Content(source=source)
    >>> mycontent.get_css(only_custom=True)
    '\\n .slide-content {\\n  float: left;\\n  background: red;\\n}\\n'
    """
    css = "\n .slide-content {\n  float: left;"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css



#!/usr/bin/env python
"""
footer.py, module definition of Footer class.
This defines the theme of the slide footer element.
"""
# modules loading
# standard library modules: these should be present in any recent python distribution
# MaTiSSe.py modules
from ..theme_element import ThemeElement
# class definition
class Footer(ThemeElement):
  """
  Object for handling the presentation theme slide footer, its attributes and methods.
  """
  def __init__(self,number,source=None):
    """
    Parameters
    ----------
    number : int
      number of header in the global numeration
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      number of header in the global numeration
    """
    self.number = number
    super(Footer,self).__init__(data_tag=r'theme_slide_footer_'+str(self.number),class_name='slide-footer_'+str(self.number))
    self.data.data['display'    ] = ['block',                       False]
    self.data.data['width'      ] = ['100%',                        False]
    self.data.data['height'     ] = ['0%',                          False]
    self.data.data['font-size'  ] = ['100%',                        False]
    self.data.data['font-family'] = ['Open Sans, Arial, sans-serif',False]
    self.data.data['overflow'   ] = ['hidden',                      False]
    if source:
      self.get(source)
      self.check_specials()
    return

  def get_options(self):
    """Method for getting the available data options.

    >>> source = '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> myfooter = Footer(number=1,source=source)
    >>> myfooter.get_options()
    '\\n\\nSlide Footer\\nmetadata = []\\nactive = True\\ndisplay = block\\nwidth = 100%\\nheight = 10%\\nfont-size = 100%\\nfont-family = Open Sans, Arial, sans-serif\\noverflow = hidden'
    """
    string = ['\n\nSlide Footer']
    string.append(self.data.get_options())
    return ''.join(string)

  def get_css(self,only_custom=False,as_list=False):
    """Method for getting css from data.

    Parameters
    ----------
    only_custom : bool, optional
      consider only (user) customized data
    as_list : bool, optional
      return a list instead of a string

    Returns
    -------
    str
      a string containing the css code of the theme if as_list = False
    list
      a list of one string containing the css code of the theme if as_list = True

    >>> source = '---theme_slide_footer_1 height = 10% ---endtheme_slide_footer_1'
    >>> myfooter = Footer(number=1,source=source)
    >>> myfooter.get_css(only_custom=True)
    '\\n .slide-footer_1 {\\n  clear: both;\\n  height: 10%;\\n}\\n'
    """
    css = "\n .slide-footer_"+str(self.number)+" {\n  clear: both;"
    css += self.data.get_css(only_custom=only_custom)
    css += "\n}\n"
    if as_list:
      return [css]
    else:
      return css
